#!/usr/bin/env python
#################################################################
##\file
#
# \note
#   Copyright (c) 2011-2012 \n
#   Fraunhofer Institute for Manufacturing Engineering
#   and Automation (IPA) \n\n
#
#################################################################
#
# \note
#   Project name: care-o-bot
# \note
#   ROS stack name: cob_calibration
# \note
#   ROS package name: cob_robot_calibration
#
# \author
#   Author: Sebastian Haug, email:sebhaug@gmail.com
# \author
#   Supervised by: Florian Weisshardt, email:florian.weisshardt@ipa.fhg.de
#
# \date Date of creation: January 2012
#
#################################################################
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer. \n
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution. \n
#     - Neither the name of the Fraunhofer Institute for Manufacturing
#       Engineering and Automation (IPA) nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission. \n
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License LGPL as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License LGPL for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License LGPL along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#
#################################################################
PKG = 'cob_robot_calibration'
import roslib; roslib.load_manifest(PKG)
import unittest

import yaml
from cob_robot_calibration import camera_yaml_updater 

FILE_IN = "test/data/test_camera_cal.yaml"
FILE_OUT = "/tmp/test_camera_cal.yaml_out"
FILE_OUT_RES_POS = "test/data/test_camera_cal.yaml_corr-result_pos"
FILE_OUT_RES_NEG = "test/data/test_camera_cal.yaml_corr-result_neg"

class TestCameraYamlUpdater(unittest.TestCase):
    def test_update_baseline_zero(self):
        # do update
        updater = camera_yaml_updater.CameraYamlUpdater(FILE_IN, FILE_OUT)
        updater.update_baseline(0.0)
        
        # compare with correct results
        self.assertTrue(self._cmp_yaml_files(FILE_OUT, FILE_IN))
        
    def test_update_baseline_positive(self):
        # do update
        updater = camera_yaml_updater.CameraYamlUpdater(FILE_IN, FILE_OUT)
        updater.update_baseline(10.0)
        
        # compare with correct results
        self.assertTrue(self._cmp_yaml_files(FILE_OUT, FILE_OUT_RES_POS))

    def test_update_baseline_negative(self):
        # do update
        updater = camera_yaml_updater.CameraYamlUpdater(FILE_IN, FILE_OUT)
        updater.update_baseline(-10.0)
        
        # compare with correct results
        self.assertTrue(self._cmp_yaml_files(FILE_OUT, FILE_OUT_RES_NEG))

    def _cmp_yaml_files(self, f1, f2):
        return yaml.load(file(f1)) == yaml.load(file(f2))

if __name__ == '__main__':
    import rosunit
    rosunit.unitrun(PKG, 'TestCameraYamlUpdater', TestCameraYamlUpdater)
